//---------------------------------------------------------------------------
// File: TMainForm.h
//
//  TDx_Input_Library
//  Example1 - Enumerating Devices
//
// Date: 21st June, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to create a DirectInput interface,
// then how to use it to enumerate available DirectInput devices,
// create a selected device, and retrieve information about the device.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
//---------------------------------------------------------------------------
#include "TDx_Input.h"
#include "TDx_InputDevice.h"
#include "TDIDevCaps.h"
#include "TDIDeviceInstance.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 or 4
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TLabel *Label1;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label4;

    TMemo *InfoMemo;
    TMemo *StatusMemo;

    TListBox *DevicesListBox;
    TListBox *DeviceCapsListBox;

    TBitBtn *CreateInterfaceButton;
    TBitBtn *EnumerateButton;
    TBitBtn *CreateSelectedButton;
    TBitBtn *RetrieveCapsButton;
    TBitBtn *DestroyActiveButton;
    TBitBtn *DestroyInterfaceButton;
    TBitBtn *ClearMessagesButton;
    TBitBtn *ExitButton;

    TDIDevCaps *DIDevCaps1;
    TDx_Input *Dx_Input1;
    TDx_InputDevice *Dx_InputDevice1;

    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall CreateInterfaceButtonClick(TObject *Sender);
    void __fastcall EnumerateButtonClick(TObject *Sender);
    void __fastcall CreateSelectedButtonClick(TObject *Sender);
    void __fastcall RetrieveCapsButtonClick(TObject *Sender);
    void __fastcall DestroyActiveButtonClick(TObject *Sender);
    void __fastcall DestroyInterfaceButtonClick(TObject *Sender);
    void __fastcall ClearMessagesButtonClick(TObject *Sender);
    void __fastcall ExitButtonClick(TObject *Sender);

    void __fastcall DevicesListBoxClick(TObject *Sender);

    void __fastcall Dx_Input1Create(TObject *Sender);
    void __fastcall Dx_Input1Destroy(TObject *Sender);
    void __fastcall Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished);

    void __fastcall Dx_InputDevice1Create(TObject *Sender);
    void __fastcall Dx_InputDevice1Destroy(TObject *Sender);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

    #define MAX_DEVICES 5

    dword       DevTypes[MAX_DEVICES];
    GUID        FFDriverGuids[MAX_DEVICES];
    GUID        InstanceGuids[MAX_DEVICES];
    AnsiString  InstanceNames[MAX_DEVICES];
    GUID        ProductGuids[MAX_DEVICES];
    AnsiString  ProductNames[MAX_DEVICES];
    word        Usages[MAX_DEVICES];
    word        UsagePages[MAX_DEVICES];
    int         NumDevices;

};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 or 4
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif
