//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Input_Library v1.4
//  Example6- Force Feedback Joysticks (extended)
//
// Date: 17th May, 2001, 18th July, 2001
// Author: Darren John Dwyer, Kerrigan Burgess
// Copyright: (c) 2001 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to control a Force Feedback Joystick,
// (or other Force Feedback Device)
//
// It helps significantly if you view and understand the previous DirectInput
// examples before you try to understand this demo.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 or 4 only
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Input"
#pragma link "TDx_InputDevice"
#pragma link "TDIDevCaps"
#pragma link "tdx_inputeffect"
#pragma link "tdidataformat"
#pragma link "tdijoystate2"
#pragma link "tdipropdword"
#pragma link "tdipropheader"
#pragma link "tdiperiodic"
#pragma link "tdienvelope"
#pragma link "tdieffect"
#pragma link "tdidevcaps"
#pragma link "tdx_input"
#pragma link "tdx_input_library_defns"
#pragma link "tdx_inputdevice"
#pragma link "tdiconstantforce"
#pragma link "tdifileeffect"
#pragma link "tdirampforce"
#pragma link "tdieffectinfo"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Input_Library_Install"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
    Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
	DeviceListBoxClear();

    if (Dx_Input1->Created)
    	Dx_Input1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateDefaultButtonClick(TObject *Sender)
{
    Msg("Creating DirectInput Interface");

    if (Dx_Input1->Created)
        Dx_Input1->Destroy();

    Dx_Input1->Create();
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyInterfaceButtonClick(TObject *Sender)
{
	if (Dx_Input1->Created)
		Dx_Input1->Destroy();
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::DeviceListBoxClear(void)
{
	for (int i=0;i<DeviceListBox->Items->Count;i++)
    	delete (TDevice *)DeviceListBox->Items->Objects[i];

    DeviceListBox->Clear();
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::EffectsListBoxClear(void)
{
	for (int i=0;i<EffectsListBox->Items->Count;i++)
    	delete (TEffect *)EffectsListBox->Items->Objects[i];

    EffectsListBox->Clear();
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumerateButtonClick(TObject *Sender)
{
    Msg("Scanning for Available Devices");

	DeviceListBoxClear();
    Dx_Input1->EnumDevices(DIDEVTYPE_JOYSTICK, NULL, DIEDFL_FORCEFEEDBACK | DIEDFL_ATTACHEDONLY );
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearMessagesButtonClick(TObject *Sender)
{
    StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished)
{
    Msg("Found a device: "+DeviceInstance->ProductName);
    int index = DeviceListBox->Items->Add( DeviceInstance->ProductName );

    TDevice* device = new TDevice;

    device->DevTypes 		= DeviceInstance->DevType;
    device->FFDriverGuids 	= DeviceInstance->FFDriver;
    device->InstanceGuids 	= DeviceInstance->Instance;
    device->InstanceNames	= DeviceInstance->InstanceName;
    device->ProductGuids	= DeviceInstance->Product;
    device->ProductNames	= DeviceInstance->ProductName;
    device->Usages			= DeviceInstance->Usage;
    device->UsagePages		= DeviceInstance->UsagePage;

	DeviceListBox->Items->Objects[index] = (TObject *)device;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
    Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Create(TObject *Sender)
{
    Msg("DirectInput Interface has been created");
    CreateDefaultButton->Enabled = false;
    EnumerateButton->Enabled = true;
    CreateSelectedButton->Enabled = false;
    DestroyInterfaceButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DeviceListBoxClick(TObject *Sender)
{
    if (DeviceListBox->Items->Count>0)
    {
        int index = DeviceListBox->ItemIndex;
        AnsiString device_name = DeviceListBox->Items->Strings[index];
        Msg("Device Selected: '"+device_name+"'");

        CreateSelectedButton->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateSelectedButtonClick(TObject *Sender)
{
    Msg("Creating Selected Device");

    if (Dx_InputDevice1->Created)
        Dx_InputDevice1->Destroy();

    int index = DeviceListBox->ItemIndex;
    if (DeviceListBox->Items->Strings[index] == "Keyboard")
    	Dx_InputDevice1->Create( GUID_SysKeyboard, Dx_Input1 );
    else if (DeviceListBox->Items->Strings[index] == "Mouse")
    	Dx_InputDevice1->Create( GUID_SysMouse, Dx_Input1 );
    else
    {
        Dx_Input1->CreateDevice( ((TDevice* )DeviceListBox->Items->Objects[index])->InstanceGuids, Dx_InputDevice1 );
        Dx_InputDevice1Create(Sender);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Destroy(TObject *Sender)
{
    Msg("DirectInput interface is about to be destroyed." );
    if (Dx_InputDevice1->Created) Dx_InputDevice1->Destroy();
	    DestroyInterfaceButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Create(TObject *Sender)
{
    Msg("Input Device has been Created");
    RetrieveCapsButton->Enabled = true;
    DestroyActiveButton->Enabled = true;

    Dx_InputDevice1->SetDataFormat( DIJoyState21 );
    Dx_InputDevice1->SetCooperativeLevel( MainForm->Handle, DISCL_EXCLUSIVE | DISCL_FOREGROUND );

    DIPropHeader1->Obj = 0;
    DIPropHeader1->How = DIPH_DEVICE;
    DIPropDWORD1->PropHeader = DIPropHeader1;
    DIPropDWORD1->Data = 0;
    Dx_InputDevice1->SetProperty( DIPROP_AUTOCENTER, DIPropDWORD1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Destroy(TObject *Sender)
{
    Msg("Input Device is about to be Destroyed");
    CreateSelectedButton->Enabled = true;
    RetrieveCapsButton->Enabled = false;
    DestroyActiveButton->Enabled = false;

    Dx_InputDevice1->Unacquire();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RetrieveCapsButtonClick(TObject *Sender)
{
    Msg("Retrieving Device Capabilities");
    DeviceCapsListBox->Items->Clear();
    if (Dx_InputDevice1->GetCapabilities( DIDevCaps1 ))
    {
        DeviceCapsListBox->Items->Add( "Axis: "+IntToStr( (int) DIDevCaps1->Axes) );
        DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
        DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
        DeviceCapsListBox->Items->Add( "FFDriverVersion: "+IntToStr( (int) DIDevCaps1->FFDriverVersion) );
        DeviceCapsListBox->Items->Add( "FFMinTimeResolution: "+IntToStr( (int) DIDevCaps1->FFMinTimeResolution) );
        DeviceCapsListBox->Items->Add( "FFSamplePeriod: "+IntToStr( (int) DIDevCaps1->FFSamplePeriod) );
        DeviceCapsListBox->Items->Add( "FirmwareRevision: "+IntToStr( (int) DIDevCaps1->FirmwareRevision) );
        DeviceCapsListBox->Items->Add( "Flags:" );
        DeviceCapsListBox->Items->AddStrings( DIDevCaps1->Flags_Strings );
        DeviceCapsListBox->Items->Add( "HardwareRevision: "+IntToStr( (int) DIDevCaps1->HardwareRevision) );
        DeviceCapsListBox->Items->Add( "POV's: "+IntToStr( (int) DIDevCaps1->POVs) );
    }
    else
    {
        DeviceCapsListBox->Items->Add( "------ Unknown ------" );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyActiveButtonClick(TObject *Sender)
{
    Msg("Destroying Active Device");
    if (Dx_InputDevice1->Created)
    	Dx_InputDevice1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateEffectButtonClick(TObject *Sender)
{
    if (EffectsListBox->ItemIndex<0)
    {
    	Msg("Please select an enumerated effect to create first");
        return;
    }
	TEffect* EnumEffect = (TEffect *)EffectsListBox->Items->Objects[EffectsListBox->ItemIndex];

    // ok, user wants to create an effect
    DIEffect1->Clear();

    // axes
    rgdwAxes[0] = DIJOFS_X;
    rgdwAxes[1] = DIJOFS_Y;

    int NumAxes = 0;
    for (int i = 0;i<EffectAxesCheckListBox->Items->Count;i++)
    {
        if (EffectAxesCheckListBox->Checked[i])
            NumAxes++;
    }
    if (NumAxes==0)
    {
        Msg("Please select atleast 1 axes for creation");
        return;
    }

    // direction
    rglDirection[0] = EffectDirectionXEdit->Text.ToInt();
    rglDirection[1] = EffectDirectionYEdit->Text.ToInt();

    switch (EffectAxesInterpComboBox->ItemIndex)
    {
        case 0:
            DIEffect1->Flags = DIEFF_CARTESIAN | DIEFF_OBJECTOFFSETS;
        break;
        case 1:
            DIEffect1->Flags = DIEFF_POLAR | DIEFF_OBJECTOFFSETS;
        break;
        case 2:
            DIEffect1->Flags = DIEFF_SPHERICAL | DIEFF_OBJECTOFFSETS;
        break;
    }

    DIEffect1->Duration = (DWORD) (EffectDurationEdit->Text.ToDouble()* DI_SECONDS);
    DIEffect1->SamplePeriod = EffectSampleEdit->Text.ToInt();
    DIEffect1->Gain = EffectGainEdit->Text.ToInt();
    DIEffect1->StartDelay = (DWORD) (EffectDelayEdit->Text.ToDouble() * DI_SECONDS);
    DIEffect1->TriggerButton = DIEB_NOTRIGGER;
    DIEffect1->TriggerRepeatInterval = 0;
    DIEffect1->AxesCount = NumAxes;
    DIEffect1->Axes = &rgdwAxes[0];
    DIEffect1->Direction = &rglDirection[0];

    // envelope
    DIEnvelope1->AttackLevel = EnvelopeAttackLevelEdit->Text.ToInt();
    DIEnvelope1->AttackTime = (DWORD) (EnvelopeAttackTimeEdit->Text.ToDouble() * DI_SECONDS);
    DIEnvelope1->FadeLevel = EnvelopeFadeLevelEdit->Text.ToInt();
    DIEnvelope1->FadeTime = (DWORD) (EnvelopeFadeTimeEdit->Text.ToDouble() * DI_SECONDS);

    DIEffect1->Envelope = DIEnvelope1;

	// figure out what type it is.
    bool ret = false;
    switch (EnumEffect->Type)
    {
    	case DIEFT_CONDITION:
            Msg("Exercise for user ;)");
            ret = true;
        break;
        case DIEFT_CONSTANTFORCE:
            DIConstantForce1->Magnitude = (DWORD) (ConstantMagnitudeEdit->Text.ToInt());

            DIEffect1->TypeSpecificParams = DIConstantForce1->Internal_DICONSTANTFORCE_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIConstantForce1->Size;
        break;
        case DIEFT_CUSTOMFORCE:
            Msg("Exercise for user ;)");
            ret = true;
        break;
        case DIEFT_PERIODIC:
            DIPeriodic1->Magnitude = PeriodicMagnitudeEdit->Text.ToInt();
            DIPeriodic1->Offset = PeriodicOffsetEdit->Text.ToInt();
            DIPeriodic1->Period = (DWORD)(PeriodicPeriodEdit->Text.ToDouble() * DI_SECONDS);
            if (DIPeriodic1->Period==0)
            	DIPeriodic1->Period = 1; // can't be zero
            DIPeriodic1->Phase = PeriodicPhaseEdit->Text.ToInt();
            if (DIPeriodic1->Phase<0)
            	DIPeriodic1->Phase = 0;
			else
            if (DIPeriodic1->Phase>35999)
            	DIPeriodic1->Phase = 35999;
            DIEffect1->TypeSpecificParams = DIPeriodic1->Internal_DIPERIODIC_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIPeriodic1->Size;
        break;
        case DIEFT_RAMPFORCE:
            DIRampForce1->Start = RampStartEdit->Text.ToInt();
            DIRampForce1->End = RampEndEdit->Text.ToInt();

            DIEffect1->TypeSpecificParams = DIRampForce1->Internal_DIRAMPFORCE_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIRampForce1->Size;
        break;
        default:
            Msg("Unsupported effect!");
            ret = true;
        break;
    }
    if (ret)
    {
    	Msg("Error in creating effect!");
        return;
    }

    // check to see if user wants effect attached to current effect
    TCreated* Created;
    int index = -1;
    if ((CreatedEffectsAttachedCheckBox->Checked) && (CreatedEffectsComboBox->Items->Count))
		Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];
    else
    {
        // create a new effect
        index = CreatedEffectsComboBox->Items->Add( "CreatedEffect"+IntToStr((int)CreatedEffectsComboBox->Items->Count));
        Created = new TCreated;
        CreatedEffectsComboBox->Items->Objects[index] = (TObject *)Created;
    }

    int index2 = Created->Add();
    Created->Data[index2]->InputEffect = new TDx_InputEffect(NULL);
    Created->Data[index2]->Name = EnumEffect->Name;
    Created->Data[index2]->InUse = true;
    Created->Data[index2]->InputEffect->OnError = Dx_Input1Error;

    if (Dx_InputDevice1->CreateEffect( EnumEffect->Guid, DIEffect1, Created->Data[index2]->InputEffect ) == S_OK)
    {
        CreatedEffectsComboBox->ItemIndex = CreatedEffectsComboBox->Items->Count-1;
        CreatedEffectsComboBoxClick(NULL);

        Msg("CreatedEffect"+IntToStr((int)CreatedEffectsComboBox->Items->Count-1));
	}
    else
    {
     	Created->Remove(index2);
        if (index>-1)
	        CreatedEffectsComboBox->Items->Delete(index);
    	Msg("Creation failed!");
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1EnumEffects(TObject *Sender, TDIEffectInfo *EffectInfo, void *Context,
    bool &Finished)
{
	int index = EffectsListBox->Items->Add(EffectInfo->Name);

    TEffect* effect = new TEffect;
    effect->Guid = EffectInfo->Guid;
    effect->Name = EffectInfo->Name;
	effect->Type = EffectInfo->EffectType;
    
    EffectsListBox->Items->Objects[index] = (TObject *)effect;
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumEffectsButtonClick(TObject *Sender)
{
	EffectsListBoxClear();
    Dx_InputDevice1->EnumEffects( NULL, DIEFT_ALL );
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayEffectButtonClick(TObject *Sender)
{
	if (CreatedEffectsComboBox->ItemIndex<0)
	{
    	Msg("Error: please select a created effect first");
		return;
    }

	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];
	for (int i=0;i<Created->Num;i++)
    {
    	if (Created->Data[i]->InUse)
        {
		    Dx_InputDevice1->Acquire();
        	Created->Data[i]->InputEffect->Start(1, 0);
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopEffectButtonClick(TObject *Sender)
{
	if (CreatedEffectsComboBox->ItemIndex<0)
	{
    	Msg("Error: please select a created effect first");
		return;
    }

    Dx_InputDevice1->Acquire();
	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];
	for (int i=0;i<Created->Num;i++)
    {
        Created->Data[i]->InputEffect->Stop();
        Created->Data[i]->InputEffect->Unload();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EffectsListBoxClick(TObject *Sender)
{
    if (!EffectsListBox->Items->Count)
        return;

	TEffect* EnumEffect = (TEffect *)EffectsListBox->Items->Objects[EffectsListBox->ItemIndex];

	// figure out what type it is.
    ResetEffectParameter(EnumEffect->Type);
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::ResetEffectParameter(DWORD Type)
{
    switch (Type)
    {
    	case DIEFT_CONDITION:
		    EffectsTypeEdit->Text = "Condition:";
            GroupBox2->Caption = EffectsTypeEdit->Text;
            EffectTypeNotebook->ActivePage = "Condition";
        break;
        case DIEFT_CONSTANTFORCE:
		    EffectsTypeEdit->Text = "Constant Force:";
            GroupBox2->Caption = EffectsTypeEdit->Text;
            EffectTypeNotebook->ActivePage = "Constant Force";
        break;
        case DIEFT_CUSTOMFORCE:
		    EffectsTypeEdit->Text = "Custom Force:";
            GroupBox2->Caption = EffectsTypeEdit->Text;
            EffectTypeNotebook->ActivePage = "Custom Force";
        break;
        case DIEFT_PERIODIC:
		    EffectsTypeEdit->Text = "Periodic:";
            GroupBox2->Caption = EffectsTypeEdit->Text;
            EffectTypeNotebook->ActivePage = "Periodic";
        break;
        case DIEFT_RAMPFORCE:
		    EffectsTypeEdit->Text = "Ramp Force:";
            GroupBox2->Caption = EffectsTypeEdit->Text;
            EffectTypeNotebook->ActivePage = "Ramp Force";
        break;
        default:
		    EffectsTypeEdit->Text = "Not Supported:";
            GroupBox2->Caption = EffectsTypeEdit->Text;
            EffectTypeNotebook->ActivePage = "Not Supported";
        break;
    }

    // reset Effect Group
    EffectAxesCheckListBox->Checked[0] = true;
    EffectAxesCheckListBox->Checked[1] = true;
    EffectAxesInterpComboBox->ItemIndex = 0;  	// cartesian
    EffectDirectionXEdit->Text = "1";
    EffectDirectionYEdit->Text = "0";
    EffectDurationEdit->Text = "0";
    EffectGainEdit->Text = "10000"; 			//  DI_FFNOMINALMAX
    EffectSampleEdit->Text = "0";
    EffectDelayEdit->Text = "0";

    // Envelope
    EnvelopeAttackLevelEdit->Text = 0;
    EnvelopeAttackTimeEdit->Text = 0;
    EnvelopeFadeLevelEdit->Text = 0;
    EnvelopeFadeTimeEdit->Text = 0;

    // Types
    ConstantMagnitudeEdit->Text = 10000;		// DI_FFNOMINALMAX

    PeriodicMagnitudeEdit->Text = 10000;		// DI_FFNOMINALMAX
    PeriodicOffsetEdit->Text = 0;
    PeriodicPeriodEdit->Text = 0;
    PeriodicPhaseEdit->Text = 0;

    RampStartEdit->Text = 0;
    RampEndEdit->Text = 0;
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadEffectButtonClick(TObject *Sender)
{
	if (OpenDialog1->Execute())
    {
        // check to see if user wants effect attached to current effect
        TCreated* Created;
        int index = -1;
        if ((CreatedEffectsAttachedCheckBox->Checked) && (CreatedEffectsComboBox->Items->Count))
            Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];
        else
        {
            // create a new effect
            index = CreatedEffectsComboBox->Items->Add( "CreatedEffect"+IntToStr((int)CreatedEffectsComboBox->Items->Count));
            Created = new TCreated;
            CreatedEffectsComboBox->Items->Objects[index] = (TObject *)Created;
        }

	    bool ret = Dx_InputDevice1->EnumEffectsInFile( OpenDialog1->FileName, Created, DIFEF_DEFAULT );
        if (!ret)
        {
	        delete CreatedEffectsComboBox->Items->Objects[index];
        	CreatedEffectsComboBox->Items->Delete(CreatedEffectsComboBox->Items->Count-1);
        }
        else
        {
        	Msg("CreatedEffect"+IntToStr((int)CreatedEffectsComboBox->Items->Count-1));
            CreatedEffectsComboBox->ItemIndex = CreatedEffectsComboBox->Items->Count-1;
            CreatedEffectsComboBoxClick(NULL);
        }
    }
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDeviceEnumEffectsInFile(TObject *Sender, TDIFileEffect *FileEffect, void *Context, bool &Finished)
{
    if (!Context)
		return;

    for (dword i=0;i<FileEffect->ArraySize;i++)
    {
		TCreated* Created = (TCreated *)Context;

        int index = Created->Add();
        Created->Data[index]->InputEffect = new TDx_InputEffect(NULL);
		Created->Data[index]->Name = FileEffect->FriendlyName[i];
        Created->Data[index]->InUse = true;
        Created->Data[index]->InputEffect->OnError = Dx_Input1Error;

        Dx_InputDevice1->CreateEffect( FileEffect->GuidEffect[i], FileEffect->Effect[i], Created->Data[index]->InputEffect );
    }
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::CreatedEffectsComboBoxClick(TObject *Sender)
{
	AttachedEffectsListBox->Clear();

	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];
	for (int i=0;i<Created->Num;i++)
	{
		int index = AttachedEffectsListBox->Items->Add(Created->Data[i]->Name);
		AttachedEffectsListBox->Checked[index] = Created->Data[i]->InUse;
    }

    // update parameters UI
    if (Created->Num)
    {
	    AttachedEffectsListBox->ItemIndex = 0;
    	AttachedEffectsListBoxClick(NULL);
    }
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::AttachedEffectsListBoxClickCheck(
      TObject *Sender)
{
	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];
	int index = AttachedEffectsListBox->ItemIndex;

	if (AttachedEffectsListBox->Checked[index])
		Created->Data[index]->InUse = true;
    else
		Created->Data[index]->InUse = false;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::AttachedEffectsListBoxClick(TObject *Sender)
{
	if (CreatedEffectsComboBox->ItemIndex<0)
    	return;

    // clear structures
    DIEffect1->Clear();
	DIEffectInfo1->Clear();
	DIEnvelope1->Clear();

    // what Attached effect are we looking at?
	int index = AttachedEffectsListBox->ItemIndex;

    // get pointer to the complete effect
	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];

	// get the effect guid
    GUID guid;
    Created->Data[index]->InputEffect->GetEffectGuid(&guid);

    // retrieve the effect info
    Dx_InputDevice1->GetEffectInfo( DIEffectInfo1, guid );

	DIEffect1->Flags 		= DIEFF_OBJECTOFFSETS | DIEFF_CARTESIAN;	// get direction in cartesian coords
    DIEffect1->AxesCount 	= 32;
    DIEffect1->Axes 		= rgdwAxes;
    DIEffect1->Direction   	= rglDirection;
	DIEffect1->Envelope		= DIEnvelope1;

    switch (DIEffectInfo1->EffectType)
    {
    	case DIEFT_CONDITION:
        break;
        case DIEFT_CONSTANTFORCE:
            DIEffect1->TypeSpecificParams = DIConstantForce1->Internal_DICONSTANTFORCE_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIConstantForce1->Size;
        break;
        case DIEFT_CUSTOMFORCE:
        break;
        case DIEFT_PERIODIC:
            DIEffect1->TypeSpecificParams = DIPeriodic1->Internal_DIPERIODIC_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIPeriodic1->Size;
        break;
        case DIEFT_RAMPFORCE:
            DIEffect1->TypeSpecificParams = DIRampForce1->Internal_DIRAMPFORCE_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIRampForce1->Size;
        break;
        default:
        break;
    }

    // retrieve the effect
	Created->Data[index]->InputEffect->GetParameters( DIEffect1, DIEP_ALLPARAMS );

    // upate
	UpdateEffectParameter();
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateEffectParameter(void)
{
    ResetEffectParameter(DIEffectInfo1->EffectType);

    // assumes we have only 2 axes
    if (DIEffect1->AxesCount==1)
    {
	    EffectAxesCheckListBox->Checked[0] = true;
    	EffectAxesCheckListBox->Checked[1] = false;

	    EffectDirectionXEdit->Text = IntToStr((int)rglDirection[0]);
        EffectDirectionYEdit->Text = "0";
    }
    else
    {
	    EffectAxesCheckListBox->Checked[0] = true;
    	EffectAxesCheckListBox->Checked[1] = true;

	    EffectDirectionXEdit->Text = IntToStr((int)rglDirection[0]);
	    EffectDirectionYEdit->Text = IntToStr((int)rglDirection[1]);
    }
    EffectAxesInterpComboBox->ItemIndex = 0;  	// cartesian

	float tmp;
    tmp = DIEffect1->Duration/(float)DI_SECONDS;
    EffectDurationEdit->Text = FloatToStr((float)tmp);

    EffectGainEdit->Text = IntToStr((int)DIEffect1->Gain);
    EffectSampleEdit->Text = IntToStr((int)DIEffect1->SamplePeriod);

    tmp = DIEffect1->StartDelay/(float)DI_SECONDS;
    EffectDelayEdit->Text = FloatToStr((float)tmp);

    // Envelope
	if (DIEffect1->Envelope)
    {
        EnvelopeAttackLevelEdit->Text = IntToStr((int)DIEffect1->Envelope->AttackLevel);
        EnvelopeFadeLevelEdit->Text = IntToStr((int)DIEffect1->Envelope->FadeLevel);

		tmp = DIEffect1->Envelope->AttackTime/(float)DI_SECONDS;
        EnvelopeAttackTimeEdit->Text = FloatToStr((float)tmp);

		tmp = DIEffect1->Envelope->FadeTime/(float)DI_SECONDS;
        EnvelopeFadeTimeEdit->Text = FloatToStr((float)tmp);
	}

    // Types
    DICONSTANTFORCE* ConstantForce;
	DIPERIODIC* Periodic;
	DIRAMPFORCE* RampForce;
    switch (DIEffectInfo1->EffectType)
    {
    	case DIEFT_CONDITION:
        break;
        case DIEFT_CONSTANTFORCE:
			ConstantForce = (DICONSTANTFORCE *)DIEffect1->TypeSpecificParams;

    		ConstantMagnitudeEdit->Text = IntToStr((int)ConstantForce->lMagnitude);
        break;
        case DIEFT_CUSTOMFORCE:
        break;
        case DIEFT_PERIODIC:
            Periodic = (DIPERIODIC *)DIEffect1->TypeSpecificParams;

            PeriodicMagnitudeEdit->Text = IntToStr((int)Periodic->dwMagnitude);
            PeriodicOffsetEdit->Text = IntToStr((int)Periodic->lOffset);
            PeriodicPhaseEdit->Text = IntToStr((int)Periodic->dwPhase);

            tmp = Periodic->dwPeriod/(float)DI_SECONDS;
            PeriodicPeriodEdit->Text = FloatToStr((float)tmp);
        break;
        case DIEFT_RAMPFORCE:
            RampForce = (DIRAMPFORCE *)DIEffect1->TypeSpecificParams;

            RampStartEdit->Text = IntToStr((int)RampForce->lStart);
            RampEndEdit->Text = IntToStr((int)RampForce->lEnd);
        break;
        default:
        break;
    }
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveEffectButtonClick(TObject *Sender)
{
	if (CreatedEffectsComboBox->ItemIndex<0)
    	return;

	if (SaveDialog1->Execute())
    {
        TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];

        DIFileEffect1->ArraySize = Created->Num;
        for (int i=0;i<Created->Num;i++)
        {
            DIFileEffect1->FriendlyName[i] = Created->Data[i]->Name;
            DIFileEffect1->GuidEffect[i] = DIEffectInfo1->Guid;
            DIFileEffect1->Effect[i] = CreateFileEffect(i);
        }

        // now write to disk
        if (Dx_InputDevice1->WriteEffectToFile(SaveDialog1->FileName, Created->Num, DIFileEffect1, DIFEF_DEFAULT))
        {
        	Msg("Effect saved to Disk!");
        }
        else
        {
        	Msg("Error saving Effect to Disk!");
        }

        // destroy any memory, when we created an effect for saving
        for (int i=0;i<Created->Num;i++)
        {
        	DIEFFECT* Effect = DIFileEffect1->Effect[i]->Internal_DIEFFECT_Ptr;
            delete Effect->rgdwAxes;
            delete Effect->rglDirection;
            delete Effect->lpEnvelope;
            delete Effect->lpvTypeSpecificParams;

            delete Effect;
        }
	}
}
//---------------------------------------------------------------------------

TDIEffect* __fastcall TMainForm::CreateFileEffect(int index)
{
	if (CreatedEffectsComboBox->ItemIndex<0)
    	return NULL;

    TDIEffect* Effect = new TDIEffect(NULL);
	TDIEnvelope* Envelope = new TDIEnvelope(NULL);

    // get pointer to the complete effect
	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];

	// get the effect guid
    GUID guid;
    Created->Data[index]->InputEffect->GetEffectGuid(&guid);

    // retrieve the effect info
    Dx_InputDevice1->GetEffectInfo( DIEffectInfo1, guid );

	Effect->Flags 		= DIEFF_OBJECTOFFSETS | DIEFF_CARTESIAN;	// get direction in cartesian coords
    Effect->AxesCount 	= 32;										// may have upto 32 axes
    Effect->Axes 		= new DWORD[32];
    Effect->Direction   = new LONG[32];

	Effect->Envelope	= Envelope;

    switch (DIEffectInfo1->EffectType)
    {
    	case DIEFT_CONDITION:
        break;
        case DIEFT_CONSTANTFORCE:
            Effect->TypeSpecificParams = new DICONSTANTFORCE;
            Effect->TypeSpecificParamsSize = sizeof(DICONSTANTFORCE);
        break;
        case DIEFT_CUSTOMFORCE:
        break;
        case DIEFT_PERIODIC:
            Effect->TypeSpecificParams = new DIPERIODIC;
            Effect->TypeSpecificParamsSize = sizeof(DIPERIODIC);
        break;
        case DIEFT_RAMPFORCE:
            Effect->TypeSpecificParams = new DIRAMPFORCE;
            Effect->TypeSpecificParamsSize = sizeof(DIRAMPFORCE);
        break;
        default:
        break;
    }

    // retrieve the effect
	if (!Created->Data[index]->InputEffect->GetParameters( Effect, DIEP_ALLPARAMS ))
    {
    	Msg("Error retrieving parameters");
    }

    return Effect;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::UpdateEffectButtonClick(TObject *Sender)
{
	if (CreatedEffectsComboBox->Items->Count<1)
    	return;

    // get pointer to the complete effect
	TCreated* Created = (TCreated *)CreatedEffectsComboBox->Items->Objects[CreatedEffectsComboBox->ItemIndex];

    // what Attached effect are we looking at?
	int index = AttachedEffectsListBox->ItemIndex;

    GUID guid;
    Created->Data[index]->InputEffect->GetEffectGuid(&guid);

    // retrieve the effect info
    Dx_InputDevice1->GetEffectInfo( DIEffectInfo1, guid );

    // axes
    rgdwAxes[0] = DIJOFS_X;
    rgdwAxes[1] = DIJOFS_Y;

    int NumAxes = 0;
    for (int i = 0;i<EffectAxesCheckListBox->Items->Count;i++)
    {
        if (EffectAxesCheckListBox->Checked[i])
            NumAxes++;
    }
    if (NumAxes==0)
    {
        Msg("Please select atleast 1 axes for update");
        return;
    }

    // direction
    rglDirection[0] = EffectDirectionXEdit->Text.ToInt();
    rglDirection[1] = EffectDirectionYEdit->Text.ToInt();

    switch (EffectAxesInterpComboBox->ItemIndex)
    {
        case 0:
            DIEffect1->Flags = DIEFF_CARTESIAN | DIEFF_OBJECTOFFSETS;
        break;
        case 1:
            DIEffect1->Flags = DIEFF_POLAR | DIEFF_OBJECTOFFSETS;
        break;
        case 2:
            DIEffect1->Flags = DIEFF_SPHERICAL | DIEFF_OBJECTOFFSETS;
        break;
    }

    DIEffect1->Duration = (DWORD) (EffectDurationEdit->Text.ToDouble()* DI_SECONDS);
    DIEffect1->SamplePeriod = EffectSampleEdit->Text.ToInt();
    DIEffect1->Gain = EffectGainEdit->Text.ToInt();
    DIEffect1->StartDelay = (DWORD) (EffectDelayEdit->Text.ToDouble() * DI_SECONDS);
    DIEffect1->TriggerButton = DIEB_NOTRIGGER;
    DIEffect1->TriggerRepeatInterval = 0;
    DIEffect1->AxesCount = NumAxes;
    DIEffect1->Axes = &rgdwAxes[0];
    DIEffect1->Direction = &rglDirection[0];

    // envelope
    DIEnvelope1->AttackLevel = EnvelopeAttackLevelEdit->Text.ToInt();
    DIEnvelope1->AttackTime = (DWORD) (EnvelopeAttackTimeEdit->Text.ToDouble() * DI_SECONDS);
    DIEnvelope1->FadeLevel = EnvelopeFadeLevelEdit->Text.ToInt();
    DIEnvelope1->FadeTime = (DWORD) (EnvelopeFadeTimeEdit->Text.ToDouble() * DI_SECONDS);

    DIEffect1->Envelope = DIEnvelope1;

	// figure out what type it is.
    bool ret = false;
    switch (DIEffectInfo1->EffectType)
    {
    	case DIEFT_CONDITION:
            Msg("Exercise for user ;)");
            ret = true;
        break;
        case DIEFT_CONSTANTFORCE:
            DIConstantForce1->Magnitude = (DWORD) (ConstantMagnitudeEdit->Text.ToInt());

            DIEffect1->TypeSpecificParams = DIConstantForce1->Internal_DICONSTANTFORCE_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIConstantForce1->Size;
        break;
        case DIEFT_CUSTOMFORCE:
            Msg("Exercise for user ;)");
            ret = true;
        break;
        case DIEFT_PERIODIC:
            DIPeriodic1->Magnitude = PeriodicMagnitudeEdit->Text.ToInt();
            DIPeriodic1->Offset = PeriodicOffsetEdit->Text.ToInt();
            DIPeriodic1->Period = (DWORD)(PeriodicPeriodEdit->Text.ToDouble() * DI_SECONDS);
            if (DIPeriodic1->Period==0)
            	DIPeriodic1->Period = 1; // can't be zero
            DIPeriodic1->Phase = PeriodicPhaseEdit->Text.ToInt();
            if (DIPeriodic1->Phase<0)
            	DIPeriodic1->Phase = 0;
			else
            if (DIPeriodic1->Phase>35999)
            	DIPeriodic1->Phase = 35999;
            DIEffect1->TypeSpecificParams = DIPeriodic1->Internal_DIPERIODIC_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIPeriodic1->Size;
        break;
        case DIEFT_RAMPFORCE:
            DIRampForce1->Start = RampStartEdit->Text.ToInt();
            DIRampForce1->End = RampEndEdit->Text.ToInt();

            DIEffect1->TypeSpecificParams = DIRampForce1->Internal_DIRAMPFORCE_Ptr;
            DIEffect1->TypeSpecificParamsSize = DIRampForce1->Size;
        break;
        default:
            Msg("Unsupported effect!");
            ret = true;
        break;
    }
    if (ret)
    {
    	Msg("Error in updating Effect!");
        return;
    }

	if (Created->Data[index]->InputEffect->SetParameters( DIEffect1, DIEP_DURATION | DIEP_GAIN | DIEP_STARTDELAY | DIEP_SAMPLEPERIOD | DIEP_ENVELOPE | DIEP_TYPESPECIFICPARAMS | DIEP_DIRECTION ))
	{
        Msg("Updated Effect!");
	}
    else
    {
    	Msg("Update failed!");
    }
}
//---------------------------------------------------------------------------

